/* eslint-disable no-magic-numbers */
import { apiPath } from '@/redux/apiPath';
import {
  ToolkitStoreWithSingleSlice,
  createStoreInstanceUsingSliceReducer,
} from '@/utils/createStoreInstanceUsingSliceReducer';
import { mockNetworkNewAPIResponse } from '@/utils/mockNetworkResponse';
import { HttpStatusCode } from 'axios';
import { LayersState } from '@/redux/layers/layers.types';
import { getLayersForModel } from '@/redux/layers/layers.thunks';
import { layersFixture } from '@/models/fixtures/layersFixture';
import { layerTextsFixture } from '@/models/fixtures/layerTextsFixture';
import { layerRectsFixture } from '@/models/fixtures/layerRectsFixture';
import { layerOvalsFixture } from '@/models/fixtures/layerOvalsFixture';
import { layerLinesFixture } from '@/models/fixtures/layerLinesFixture';
import layersReducer from './layers.slice';

const mockedAxiosClient = mockNetworkNewAPIResponse();

describe('layers thunks', () => {
  let store = {} as ToolkitStoreWithSingleSlice<LayersState>;
  beforeEach(() => {
    store = createStoreInstanceUsingSliceReducer('layers', layersReducer);
  });

  describe('getLayersForModel', () => {
    it('should return data when data response from API is valid', async () => {
      mockedAxiosClient.onGet(apiPath.getLayers(1)).reply(HttpStatusCode.Ok, layersFixture);
      mockedAxiosClient
        .onGet(apiPath.getLayerTexts(1, layersFixture.content[0].id))
        .reply(HttpStatusCode.Ok, layerTextsFixture);
      mockedAxiosClient
        .onGet(apiPath.getLayerRects(1, layersFixture.content[0].id))
        .reply(HttpStatusCode.Ok, layerRectsFixture);
      mockedAxiosClient
        .onGet(apiPath.getLayerOvals(1, layersFixture.content[0].id))
        .reply(HttpStatusCode.Ok, layerOvalsFixture);
      mockedAxiosClient
        .onGet(apiPath.getLayerLines(1, layersFixture.content[0].id))
        .reply(HttpStatusCode.Ok, layerLinesFixture);

      const { payload } = await store.dispatch(getLayersForModel(1));
      expect(payload).toEqual({
        layers: [
          {
            details: layersFixture.content[0],
            texts: layerTextsFixture.content,
            rects: layerRectsFixture.content,
            ovals: layerOvalsFixture.content,
            lines: layerLinesFixture.content,
          },
        ],
        layersVisibility: {
          [layersFixture.content[0].layerId]: layersFixture.content[0].visible,
        },
      });
    });

    it('should return undefined when data response from API is not valid ', async () => {
      mockedAxiosClient
        .onGet(apiPath.getLayers(1))
        .reply(HttpStatusCode.Ok, { randomProperty: 'randomValue' });

      const { payload } = await store.dispatch(getLayersForModel(1));
      expect(payload).toEqual(undefined);
    });
  });
});
